searchData={"items":[{"type":"module","title":"Inertia","doc":"The Inertia.js adapter for Elixir/Phoenix applications.","ref":"Inertia.html"},{"type":"module","title":"Inertia.Controller","doc":"Controller functions for rendering Inertia.js responses.","ref":"Inertia.Controller.html"},{"type":"function","title":"Inertia.Controller.assign_errors/2","doc":"Assigns errors to the Inertia page data. This helper accepts any data that\nimplements the `Inertia.Errors` protocol. By default, this library implements\nerror serializers for `Ecto.Changeset` and bare maps.\n\nIf you are serializing your own errors maps, they should take the following shape:\n\n    %{\n      \"name\" => \"Name is required\",\n      \"password\" => \"Password must be at least 5 characters\",\n      \"team.name\" => \"Team name is required\",\n    }\n\nWhen assigning a changeset, you may optionally pass a message-generating function\nto use when traversing errors. See [`Ecto.Changeset.traverse_errors/2`](https://hexdocs.pm/ecto/Ecto.Changeset.html#traverse_errors/2)\nfor more information about the message function.\n\n    defp default_msg_func({msg, opts}) do\n      Enum.reduce(opts, msg, fn {key, value}, acc ->\n        String.replace(acc, \"%{#{key}}\", fn _ -> to_string(value) end)\n      end)\n    end\n\nThis default implementation performs a simple string replacement for error\nmessage containing variables, like `count`. For example, given the following\nerror:\n\n    {\"should be at least %{count} characters\", [count: 3, validation: :length, min: 3]}\n\nThe generated description would be \"should be at least 3 characters\". If you would\nprefer to use the `Gettext` module for pluralizing and localizing error messages, you\ncan override the message function:\n\n    conn\n    |> assign_errors(changeset, fn {msg, opts} ->\n      if count = opts[:count] do\n        Gettext.dngettext(MyAppWeb.Gettext, \"errors\", msg, msg, count, opts)\n      else\n        Gettext.dgettext(MyAppWeb.Gettext, \"errors\", msg, opts)\n      end\n    end)","ref":"Inertia.Controller.html#assign_errors/2"},{"type":"function","title":"Inertia.Controller.assign_errors/3","doc":"","ref":"Inertia.Controller.html#assign_errors/3"},{"type":"function","title":"Inertia.Controller.assign_prop/3","doc":"Assigns a prop value to the Inertia page data.","ref":"Inertia.Controller.html#assign_prop/3"},{"type":"function","title":"Inertia.Controller.camelize_props/1","doc":"Enable (or disable) automatic conversion of prop keys from snake case (e.g.\n`inserted_at`), which is conventional in Elixir, to camel case (e.g.\n`insertedAt`), which is conventional in JavaScript.","ref":"Inertia.Controller.html#camelize_props/1"},{"type":"function","title":"Examples - Inertia.Controller.camelize_props/1","doc":"Using `camelize_props` here will convert `first_name` to `firstName` in the\nresponse props.\n\n    conn\n    |> assign_prop(:first_name, \"Bob\")\n    |> camelize_props()\n    |> render_inertia(\"Home\")\n\nYou may also pass a boolean to the `camelize_props` function (to override any\npreviously-set or globally-configured value):\n\n    conn\n    |> assign_prop(:first_name, \"Bob\")\n    |> camelize_props(false)\n    |> render_inertia(\"Home\")","ref":"Inertia.Controller.html#camelize_props/1-examples"},{"type":"function","title":"Inertia.Controller.camelize_props/2","doc":"","ref":"Inertia.Controller.html#camelize_props/2"},{"type":"function","title":"Inertia.Controller.clear_history/1","doc":"Instuct the client-side to clear the history.","ref":"Inertia.Controller.html#clear_history/1"},{"type":"function","title":"Inertia.Controller.clear_history/2","doc":"","ref":"Inertia.Controller.html#clear_history/2"},{"type":"function","title":"Inertia.Controller.encrypt_history/1","doc":"Instuct the client-side to encrypt history for this page.","ref":"Inertia.Controller.html#encrypt_history/1"},{"type":"function","title":"Inertia.Controller.encrypt_history/2","doc":"","ref":"Inertia.Controller.html#encrypt_history/2"},{"type":"function","title":"Inertia.Controller.force_inertia_redirect/2","doc":"Forces the Inertia.js client side to perform a redirect. This can be used as a\nplug or inline when building a response.\n\nThis plug modifies the response to be a 409 Conflict response and include the\ndestination URL in the `x-inertia-location` header, which will cause the\nInertia client to perform a `window.location = url` visit.\n\n**Note**: we automatically convert regular external redirects (via the Phoenix\n`redirect` helper), but this function is useful if you need to force redirect\nto a non-external route that is not handled by Inertia.\n\nSee https://inertiajs.com/redirects#external-redirects","ref":"Inertia.Controller.html#force_inertia_redirect/2"},{"type":"function","title":"Examples - Inertia.Controller.force_inertia_redirect/2","doc":"conn\n    |> force_inertia_redirect()\n    |> redirect(to: \"/non-inertia-powered-page\")","ref":"Inertia.Controller.html#force_inertia_redirect/2-examples"},{"type":"function","title":"Inertia.Controller.inertia_always/1","doc":"Marks a prop value as \"always included\", which means it will be included in\nthe props on initial page load and subsequent partial loads (even when it's\nnot explicitly requested).","ref":"Inertia.Controller.html#inertia_always/1"},{"type":"function","title":"Inertia.Controller.inertia_defer/1","doc":"Marks that a prop should fetched immediately after the page is loaded on the client-side.","ref":"Inertia.Controller.html#inertia_defer/1"},{"type":"function","title":"Inertia.Controller.inertia_defer/2","doc":"","ref":"Inertia.Controller.html#inertia_defer/2"},{"type":"function","title":"Inertia.Controller.inertia_merge/1","doc":"Marks that a prop should be merged with existing data on the client-side.","ref":"Inertia.Controller.html#inertia_merge/1"},{"type":"function","title":"Inertia.Controller.inertia_optional/1","doc":"Marks a prop value as optional, which means it will only get evaluated if\nexplicitly requested in a partial reload.\n\nOptional props will _only_ be included the when explicitly requested in a\npartial reload. If you want to include the prop on first visit, you'll want to\nuse a bare anonymous function or named function reference instead.\n\n    conn\n    # ALWAYS included on first visit...\n    # OPTIONALLY included on partial reloads...\n    # ALWAYS evaluated...\n    |> assign_prop(:cheap_thing, cheap_thing())\n\n    # ALWAYS included on first visit...\n    # OPTIONALLY included on partial reloads...\n    # ONLY evaluated when needed...\n    |> assign_prop(:expensive_thing, fn -> calculate_thing() end)\n    |> assign_prop(:another_expensive_thing, &calculate_another_thing/0)\n\n    # NEVER included on first visit...\n    # OPTIONALLY included on partial reloads...\n    # ONLY evaluated when needed...\n    |> assign_prop(:super_expensive_thing, inertia_optional(fn -> calculate_thing() end))","ref":"Inertia.Controller.html#inertia_optional/1"},{"type":"function","title":"Inertia.Controller.inertia_response?/1","doc":"Determines if a response has been rendered with Inertia.","ref":"Inertia.Controller.html#inertia_response?/1"},{"type":"function","title":"Inertia.Controller.preserve_case/1","doc":"Prevents auto-transformation of a prop key to camel-case (when\n`camelize_props` is enabled).","ref":"Inertia.Controller.html#preserve_case/1"},{"type":"function","title":"Example - Inertia.Controller.preserve_case/1","doc":"conn\n    |> assign_prop(preserve_case(:this_will_not_be_camelized), \"value\")\n    |> assign_prop(:this_will_be_camelized, \"another_value\")\n    |> camelize_props()\n    |> render_inertia(\"Home\")\n\nYou can also use this helper inside of nested props:\n\n    conn\n    |> assign_prop(:user, %{\n      preserve_case(:this_will_not_be_camelized) => \"value\",\n      this_will_be_camelized: \"another_value\"\n    })\n    |> camelize_props()\n    |> render_inertia(\"Home\")","ref":"Inertia.Controller.html#preserve_case/1-example"},{"type":"function","title":"Inertia.Controller.render_inertia/2","doc":"Renders an Inertia response.","ref":"Inertia.Controller.html#render_inertia/2"},{"type":"function","title":"Options - Inertia.Controller.render_inertia/2","doc":"- `ssr`: whether to server-side render the response (see the docs on\n  \"Server-side rendering\" in the README for more information on setting this\n  up). Defaults to the globally-configured value, or `false` if no global\n  config is specified.","ref":"Inertia.Controller.html#render_inertia/2-options"},{"type":"function","title":"Examples - Inertia.Controller.render_inertia/2","doc":"conn\n    |> assign_prop(:user_id, 1)\n    |> render_inertia(\"SettingsPage\")\n\nYou may pass additional props as map for the third argument:\n\n    conn\n    |> assign_prop(:user_id, 1)\n    |> render_inertia(\"SettingsPage\", %{name: \"Bob\"})\n\nYou may also pass options for the last positional argument:\n\n    conn\n    |> assign_prop(:user_id, 1)\n    |> render_inertia(\"SettingsPage\", ssr: true)\n\n    conn\n    |> assign_prop(:user_id, 1)\n    |> render_inertia(\"SettingsPage\", %{name: \"Bob\"}, ssr: true)","ref":"Inertia.Controller.html#render_inertia/2-examples"},{"type":"function","title":"Inertia.Controller.render_inertia/3","doc":"","ref":"Inertia.Controller.html#render_inertia/3"},{"type":"function","title":"Inertia.Controller.render_inertia/4","doc":"","ref":"Inertia.Controller.html#render_inertia/4"},{"type":"opaque","title":"Inertia.Controller.always/0","doc":"","ref":"Inertia.Controller.html#t:always/0"},{"type":"opaque","title":"Inertia.Controller.defer/0","doc":"","ref":"Inertia.Controller.html#t:defer/0"},{"type":"opaque","title":"Inertia.Controller.merge/0","doc":"","ref":"Inertia.Controller.html#t:merge/0"},{"type":"opaque","title":"Inertia.Controller.optional/0","doc":"","ref":"Inertia.Controller.html#t:optional/0"},{"type":"opaque","title":"Inertia.Controller.preserved_prop_key/0","doc":"","ref":"Inertia.Controller.html#t:preserved_prop_key/0"},{"type":"type","title":"Inertia.Controller.prop_key/0","doc":"","ref":"Inertia.Controller.html#t:prop_key/0"},{"type":"type","title":"Inertia.Controller.raw_prop_key/0","doc":"","ref":"Inertia.Controller.html#t:raw_prop_key/0"},{"type":"type","title":"Inertia.Controller.render_opt/0","doc":"","ref":"Inertia.Controller.html#t:render_opt/0"},{"type":"type","title":"Inertia.Controller.render_opts/0","doc":"","ref":"Inertia.Controller.html#t:render_opts/0"},{"type":"protocol","title":"Inertia.Errors","doc":"Converts a value to Inertia.js-compatible [validation\nerrors](https://inertiajs.com/validation).\n\nThis protocol allows you to transform various error structures into the format\nexpected by Inertia.js for client-side validation. The protocol converts error\nstructures into a flat map where keys represent field paths and values are error\nmessages.","ref":"Inertia.Errors.html"},{"type":"protocol","title":"Built-in Implementations - Inertia.Errors","doc":"This library includes default implementations for:\n\n* `Ecto.Changeset` - Converts changeset errors to Inertia-compatible format\n* `Map` - Validates and passes through properly formatted error maps","ref":"Inertia.Errors.html#module-built-in-implementations"},{"type":"protocol","title":"Usage with Ecto.Changeset - Inertia.Errors","doc":"```elixir\n# In your controller action\ndef create(conn, %{\"post\" => post_params}) do\n  case Posts.create(post_params)\n    # Handle successful validation\n    {:ok, post} ->\n      redirect(conn, to: ~p\"/posts/#{post}\")\n\n    # Convert changeset errors and share them with Inertia\n    {:error, changeset} ->\n      conn\n      |> assign_errors(changeset)\n      |> redirect(conn, to: ~p\"/posts/new\")\n  end\nend\n```\n\nThe `assign_errors/2` function is a convenience helper provided by `Inertia.Controller`\nthat internally uses `Inertia.Errors.to_errors/1` to serialize the changeset errors\nand share them with the Inertia page under the `errors` key.","ref":"Inertia.Errors.html#module-usage-with-ecto-changeset"},{"type":"protocol","title":"Custom Error Formatting - Inertia.Errors","doc":"You can provide a custom message formatting function as the second argument to\n`to_errors/2`:\n\n```elixir\nInertia.Errors.to_errors(changeset, fn {msg, opts} ->\n  # Custom error message formatting logic\n  Gettext.dgettext(MyApp.Gettext, \"errors\", msg, opts)\nend)\n```","ref":"Inertia.Errors.html#module-custom-error-formatting"},{"type":"protocol","title":"Implementing for Custom Types - Inertia.Errors","doc":"You can implement this protocol for your own error types:\n\n```elixir\ndefimpl Inertia.Errors, for: MyApp.ValidationError do\n  def to_errors(validation_error) do\n    # Convert your custom error structure to a map of field paths to error\n    # messages\n    %{\n      \"field_name\" => validation_error.message,\n      \"nested.field\" => \"Another error message\"\n    }\n  end\n\n  def to_errors(validation_error, _msg_func) do\n    # Custom implementation with message function\n    to_errors(validation_error)\n  end\nend\n```","ref":"Inertia.Errors.html#module-implementing-for-custom-types"},{"type":"function","title":"Inertia.Errors.to_errors/1","doc":"","ref":"Inertia.Errors.html#to_errors/1"},{"type":"function","title":"Inertia.Errors.to_errors/2","doc":"","ref":"Inertia.Errors.html#to_errors/2"},{"type":"type","title":"Inertia.Errors.t/0","doc":"All the types that implement this protocol.","ref":"Inertia.Errors.html#t:t/0"},{"type":"module","title":"Inertia.HTML","doc":"HTML components for Inertia views.","ref":"Inertia.HTML.html"},{"type":"function","title":"Inertia.HTML.inertia_head/1","doc":"","ref":"Inertia.HTML.html#inertia_head/1"},{"type":"function","title":"Attributes - Inertia.HTML.inertia_head/1","doc":"* `content` (`:list`) (required) - The list of tags to inject into the `head` tag.","ref":"Inertia.HTML.html#inertia_head/1-attributes"},{"type":"function","title":"Inertia.HTML.inertia_title/1","doc":"Renders a ` ` tag that includes the `inertia` attribute needed for the\nInertia.js client-side to subsequently manage the page title.\n\nThe content you provide to this component will only apply to the initial\nserver-rendered response. You will also need to use the [client-side ` `\ncomponent](https://inertiajs.com/title-and-meta) in your Inertia page\ncomponents to be sure the the page title is updated when internal navigation\noccurs.\n\nIf you are planning to manage page titles from the server-side, you may find\nit useful to expose the `page_title` via regular assigns (so your\n`<.inertia_title>` can use it) AND via Inertia page props (so your client-side\n` ` use it):\n\n    def index(conn, _params)\n      page_title = \"Your page title\"\n\n      conn\n      |> assign(:page_title, page_title)\n      |> assign_prop(:page_title, page_title)\n      |> render_inertia(\"YourPage\")\n    end","ref":"Inertia.HTML.html#inertia_title/1"},{"type":"function","title":"Attributes - Inertia.HTML.inertia_title/1","doc":"* `prefix` (`:string`) - A prefix added before the content of `inner_block`. Defaults to `nil`.\n* `suffix` (`:string`) - A suffix added after the content of `inner_block`. Defaults to `nil`.","ref":"Inertia.HTML.html#inertia_title/1-attributes"},{"type":"function","title":"Slots - Inertia.HTML.inertia_title/1","doc":"* `inner_block` (required) - Content rendered inside the `title` tag.","ref":"Inertia.HTML.html#inertia_title/1-slots"},{"type":"module","title":"Inertia.Plug","doc":"The plug module for detecting Inertia.js requests.","ref":"Inertia.Plug.html"},{"type":"function","title":"Inertia.Plug.call/2","doc":"","ref":"Inertia.Plug.html#call/2"},{"type":"function","title":"Inertia.Plug.init/1","doc":"","ref":"Inertia.Plug.html#init/1"},{"type":"module","title":"Inertia.SSR","doc":"A supervisor that provides SSR support for Inertia views. This module is\nresponsible for starting a pool of Node.js processes that can run the SSR\nrendering function for your application.","ref":"Inertia.SSR.html"},{"type":"function","title":"Inertia.SSR.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Inertia.SSR.html#child_spec/1"},{"type":"function","title":"Inertia.SSR.start_link/1","doc":"Starts the SSR supervisor and accompanying Node.js workers.","ref":"Inertia.SSR.html#start_link/1"},{"type":"function","title":"Options - Inertia.SSR.start_link/1","doc":"- `:path` - (required) the path to the directory where your `ssr.js` file lives.\n- `:module` - (optional) the name of the Node.js module file. Defaults to \"ssr\".\n- `:pool_size` - (optional) the number of Node.js workers. Defaults to 4.","ref":"Inertia.SSR.html#start_link/1-options"},{"type":"module","title":"Inertia.Testing","doc":"Helpers for testing Inertia responses.","ref":"Inertia.Testing.html"},{"type":"function","title":"Inertia.Testing.inertia_component/1","doc":"Fetches the Inertia component (if applicable) for the current request.","ref":"Inertia.Testing.html#inertia_component/1"},{"type":"function","title":"Example - Inertia.Testing.inertia_component/1","doc":"use MyAppWeb.ConnCase\n\n    import Inertia.Testing\n\n    describe \"GET /\" do\n      test \"renders the home page\", %{conn: conn} do\n        conn = get(\"/\")\n        assert inertia_component(conn) == \"Home\"\n      end\n    end","ref":"Inertia.Testing.html#inertia_component/1-example"},{"type":"function","title":"Inertia.Testing.inertia_errors/1","doc":"Fetches the Inertia errors (if applicable) for the current request.\n\nIf there are errors available in the current page props, they will be returned.\nOtherwise, errors that have been stored in the session will be retrieved.","ref":"Inertia.Testing.html#inertia_errors/1"},{"type":"function","title":"Example - Inertia.Testing.inertia_errors/1","doc":"use MyAppWeb.ConnCase\n\n    import Inertia.Testing\n\n    describe \"POST /users\" do\n      test \"fails when name empty\", %{conn: conn} do\n        conn = post(\"/users\", %{\"name\" => \"\"})\n\n        assert %{user: %{id: 1}} = inertia_props(conn)\n        assert redirected_to(conn) == ~p\"/users\"\n        assert inertia_errors(conn) == %{\"name\" => \"can't be blank\"}\n      end\n    end","ref":"Inertia.Testing.html#inertia_errors/1-example"},{"type":"function","title":"Inertia.Testing.inertia_props/1","doc":"Fetches the Inertia props (if applicable) for the current request.","ref":"Inertia.Testing.html#inertia_props/1"},{"type":"function","title":"Example - Inertia.Testing.inertia_props/1","doc":"use MyAppWeb.ConnCase\n\n    import Inertia.Testing\n\n    describe \"GET /\" do\n      test \"renders the home page\", %{conn: conn} do\n        conn = get(\"/\")\n        assert %{user: %{id: 1}} = inertia_props(conn)\n      end\n    end","ref":"Inertia.Testing.html#inertia_props/1-example"},{"type":"exception","title":"Inertia.SSR.RenderError","doc":"","ref":"Inertia.SSR.RenderError.html"},{"type":"extras","title":"Inertia.js Phoenix Adapter","doc":"# Inertia.js Phoenix Adapter [![Hex Package](https://img.shields.io/hexpm/v/inertia)](https://hex.pm/packages/inertia) [![Hex Docs](https://img.shields.io/badge/docs-green)](https://hexdocs.pm/inertia/readme.html)\n\nThe official Elixir/Phoenix adapter for [Inertia.js](https://inertiajs.com/).","ref":"readme.html"},{"type":"extras","title":"Table of Contents - Inertia.js Phoenix Adapter","doc":"- [Installation](#installation)\n- [Rendering responses](#rendering-responses)\n- [Setting up the client-side](#setting-up-the-client-side)\n- [Lazy data evaluation](#lazy-data-evaluation)\n- [Deferred props](#deferred-props)\n- [Merge props](#merge-props)\n- [Shared data](#shared-data)\n- [Validations](#validations)\n- [Flash messages](#flash-messages)\n- [CSRF protection](#csrf-protection)\n- [History](#history)\n- [Testing](#testing)\n- [Server-side rendering](#server-side-rendering)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Installation - Inertia.js Phoenix Adapter","doc":"The package can be installed by adding `inertia` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:inertia, \"~> 2.4.0\"}\n  ]\nend\n```\n\nAdd your desired configuration in your `config.exs` file:\n\n```elixir\n# config/config.exs\n\nconfig :inertia,\n  # The Phoenix Endpoint module for your application. This is used for building\n  # asset URLs to compute a unique version hash to track when something has\n  # changed (and a reload is required on the frontend).\n  endpoint: MyAppWeb.Endpoint,\n\n  # An optional list of static file paths to track for changes. You'll generally\n  # want to include any JavaScript assets that may require a page refresh when\n  # modified.\n  static_paths: [\"/assets/app.js\"],\n\n  # The default version string to use (if you decide not to track any static\n  # assets using the `static_paths` config). Defaults to \"1\".\n  default_version: \"1\",\n\n  # Enable automatic conversion of prop keys from snake case (e.g. `inserted_at`),\n  # which is conventional in Elixir, to camel case (e.g. `insertedAt`), which is\n  # conventional in JavaScript. Defaults to `false`.\n  camelize_props: false,\n\n  # Instruct the client side whether to encrypt the page object in the window history\n  # state. This can also be set/overridden on a per-request basis, using the `encrypt_history`\n  # controller helper. Defaults to `false`.\n  history: [encrypt: false],\n\n  # Enable server-side rendering for page responses (requires some additional setup,\n  # see instructions below). Defaults to `false`.\n  ssr: false,\n\n  # Whether to raise an exception when server-side rendering fails (only applies\n  # when SSR is enabled). Defaults to `true`.\n  #\n  # Recommended: enable in non-production environments and disable in production,\n  # so that SSR failures will not cause 500 errors (but instead will fallback to\n  # CSR).\n  raise_on_ssr_failure: config_env() != :prod\n```\n\nThis library includes a few modules to help render Inertia responses:\n\n- [`Inertia.Plug`](https://hexdocs.pm/inertia/Inertia.Plug.html): a plug for detecting Inertia.js requests and preparing the connection accordingly.\n- [`Inertia.Controller`](https://hexdocs.pm/inertia/Inertia.Controller.html): controller functions for rendering Inertia.js-compatible responses.\n- [`Inertia.HTML`](https://hexdocs.pm/inertia/Inertia.HTML.html): HTML components for Inertia-powered views.\n\nTo get started, import `Inertia.Controller` in your controller helper and `Inertia.HTML` in your html helper:\n\n```diff\n  # lib/my_app_web.ex\n  defmodule MyAppWeb do\n    def controller do\n      quote do\n        use Phoenix.Controller, namespace: MyAppWeb\n\n+       import Inertia.Controller\n      end\n    end\n\n    def html do\n      quote do\n        use Phoenix.Component\n\n+       import Inertia.HTML\n      end\n    end\n  end\n```\n\nThen, install the plug in your browser pipeline:\n\n```diff\n  # lib/my_app_web/router.ex\n  defmodule MyAppWeb.Router do\n    use MyAppWeb, :router\n\n    pipeline :browser do\n      plug :accepts, [\"html\"]\n\n+     plug Inertia.Plug\n    end\n  end\n```\n\nNext, replace the title tag in your layout with the `<.inertia_title>` component, so that the client-side library will keep the title in sync, and add the `<.inertia_head>` component:\n\n```diff\n  # lib/my_app_web/components/layouts/root.html.heex\n  <!DOCTYPE html>\n   \n     \n-     <.live_title>{@page_title}</.live_title>\n+     <.inertia_title>{@page_title}</.inertia_title>\n+     <.inertia_head content={@inertia_head} />\n     \n```\n\nYou're now ready to start rendering inertia responses!","ref":"readme.html#installation"},{"type":"extras","title":"Rendering responses - Inertia.js Phoenix Adapter","doc":"Rendering an Inertia.js response looks like this:\n\n```elixir\ndefmodule MyAppWeb.ProfileController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    conn\n    |> assign_prop(:text, \"Hello world\")\n    |> render_inertia(\"ProfilePage\")\n  end\nend\n```\n\nThe `assign_prop` function allows you to define props that should be passed in to the component. The `render_inertia` function accepts the conn, the name of the component to render, and an optional map containing more initial props to pass to the page component.\n\nThis action will render an HTML page containing a ` ` element with the name of the component and the initial props, following Inertia.js conventions. On subsequent requests dispatched by the Inertia.js client library, this action will return a JSON response with the data necessary for rendering the page.\n\nIf you want to automatically convert your prop keys from snake case (conventional in Elixir) to camel case to keep with JavaScript conventions (e.g. `first_name` to `firstName`), you can configure that globally or enable/disable it on a per-request basis.\n\n```elixir\nimport Config\n\nconfig :inertia,\n  endpoint: MyAppWeb.Endpoint,\n  camelize_props: true\n```\n\n```elixir\ndefmodule MyAppWeb.ProfileController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    conn\n    |> assign_prop(:first_name, \"Bob\")\n    |> camelize_props()\n    |> render_inertia(\"ProfilePage\")\n  end\nend\n```","ref":"readme.html#rendering-responses"},{"type":"extras","title":"Setting up the client-side - Inertia.js Phoenix Adapter","doc":"The [Inertia.js docs](https://inertiajs.com/client-side-setup) provide a good general walk-through on how to setup your JavaScript assets to boot your Inertia app. If you're new to Inertia, we recommend checking that out to familiarize yourself with how it all works. Here we'll provide some guidance on getting your Phoenix app with esbuild configured for basic client-side rendering (and further down, we'll delve into server-side rendering).\n\nTo get started, install the Inertia.js library for the frontend framework of your choice. In these instructions we'll use React, but the process is similar for other Inertia-compatible frameworks, like Vue or Svelte.\n\n```\ncd assets\nnpm install @inertiajs/react react react-dom\n```\n\nReplace the contents of your `app.js` file with the Inertia boot function and rename it to `app.jsx` (since we are using JSX).\n\n```javascript\n// assets/js/app.jsx\n\nimport React from \"react\";\nimport axios from \"axios\";\n\nimport { createInertiaApp } from \"@inertiajs/react\";\nimport { createRoot } from \"react-dom/client\";\n\naxios.defaults.xsrfHeaderName = \"x-csrf-token\";\n\ncreateInertiaApp({\n  resolve: async (name) => {\n    return await import(`./pages/${name}.jsx`);\n  },\n  setup({ App, el, props }) {\n    createRoot(el).render( );\n  },\n});\n```\n\nThe example above assumes your pages live in the `assets/js/pages` directory and have a default export with page component, like this:\n\n```javascript\n// assets/js/pages/Dashboard.jsx\n\nimport React from \"react\";\n\nconst Dashboard = () => {\n  return (\n     \n      {/* ... page contents ...*/}\n     \n  );\n}\n\nexport default Dashboard;\n```\n\nNext, make some adjustments to your esbuild config:\n\n- Ensure the version is >= 0.19.0 (this is required for glob-style imports for your pages)\n- Update your entrypoint filename to the correct `.jsx` extension\n- Ensure your build `--target` is at least `es2020`\n\n```elixir\n# config/config.exs\n\nconfig :esbuild,\n  version: \"0.21.5\",\n  my_app: [\n    args: ~w(js/app.jsx --bundle --target=es2020 --outdir=../priv/static/assets --external:/fonts/* --external:/images/*),\n    cd: Path.expand(\"../assets\", __DIR__),\n    env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n  ]\n```\n\nIf you updated your esbuild version, you'll need to run `mix esbuild.install` to fetch the new version.\n\nEsbuild also supports code-splitting, which can be useful for larger applications. To enable it, you'll need to:\n\n- Set the `format` as [`esm`](https://esbuild.github.io/api/#format-esm)\n- Add the [`--splitting`](https://esbuild.github.io/api/#splitting) flag\n- Optionally, set the [`chunk-names`](https://esbuild.github.io/api/#chunk-names) flag to customize the output filenames\n\n```diff\n  # config/config.exs\n\n  config :esbuild,\n    version: \"0.21.5\",\n    my_app: [\n-     args: ~w(js/app.jsx --bundle --target=es2020 --outdir=../priv/static/assets --external:/fonts/* --external:/images/*),\n+     args: ~w(js/app.jsx --bundle --chunk-names=chunks/[name]-[hash] --splitting --format=esm --target=es2020 --outdir=../priv/static/assets --external:/fonts/* --external:/images/*),\n      cd: Path.expand(\"../assets\", __DIR__),\n      env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n    ]\n```\n\nAfter that, we need to update our root layout to load the JavaScript bundle as an ESM module (by changing the `type` attribute from `text/javascript` to `module`):\n\n```diff\n  # lib/my_app_web/components/layouts/root.html.eex\n\n-    \n+    \n```\n\n> [!NOTE]\n> ESM code splitting requires modern browser support.\n> While most current browsers support ESM modules, you should verify compatibility requirements with your target audience.\n> You can read more about how code-splitting works with esbuild in the [official documentation](https://esbuild.github.io/api/#chunk-names).","ref":"readme.html#setting-up-the-client-side"},{"type":"extras","title":"Lazy data evaluation - Inertia.js Phoenix Adapter","doc":"If you have expensive data for your props that may not always be required (that is, if you plan to use [partial reloads](https://inertiajs.com/partial-reloads)), you can wrap your expensive computation in a function and pass the function reference when setting your Inertia props. You may use either an anonymous function (or named function reference) and optionally wrap it with the `Inertia.Controller.inertia_optional/1` function.\n\n> [!NOTE]\n> `inertia_optional` props will _only_ be included the when explicitly requested in a partial\n> reload. If you want to include the prop on first visit, you'll want to use a\n> bare anonymous function or named function reference instead. See below for\n> examples of how prop assignment behaves.\n\nHere are some specific examples of how the methods of lazy data evaluation differ:\n\n```elixir\nconn\n# ALWAYS included on first visit...\n# OPTIONALLY included on partial reloads...\n# ALWAYS evaluated...\n|> assign_prop(:cheap_thing, cheap_thing())\n\n# ALWAYS included on first visit...\n# OPTIONALLY included on partial reloads...\n# ONLY evaluated when needed...\n|> assign_prop(:expensive_thing, fn -> calculate_thing() end)\n|> assign_prop(:another_expensive_thing, &calculate_another_thing/0)\n\n# NEVER included on first visit...\n# OPTIONALLY included on partial reloads...\n# ONLY evaluated when needed...\n|> assign_prop(:super_expensive_thing, inertia_optional(fn -> calculate_thing() end))\n```","ref":"readme.html#lazy-data-evaluation"},{"type":"extras","title":"Deferred props - Inertia.js Phoenix Adapter","doc":"**Requires Inertia v2.x on the client-side**.\n\nIf you have expensive data that you'd like to automatically fetch (from the client-side via an async background request) after the page is initially rendered, you can mark the prop as deferred:\n\n```elixir\nconn\n|> assign_prop(:expensive_thing, inertia_defer(fn -> calculate_thing() end))\n```\n\nThe `inertia_defer/1` helper accepts a function argument in the first position. You may optionally use the `inertia_defer/2` helper, which accepts a \"group\" name in the second position:\n\n```elixir\nconn\n|> assign_prop(:expensive_thing, inertia_defer(fn -> calculate_thing() end, \"dashboard\"))\n```\n\nIf no group names are specified, then the client-side will issue a single async request to fetch all the deferred props. If there are multiple group names, then the client-side will issue one async request per group instead. This is useful if you have some very expensive data that you'd prefer fetch in parallel alongside other expensive data.","ref":"readme.html#deferred-props"},{"type":"extras","title":"Merge props - Inertia.js Phoenix Adapter","doc":"**Requires Inertia v2.x on the client-side**.\n\nIf you have prop data that should get merged with the existing data on the client-side on subsequent requests (for example, an array of paginated data being presented in an \"infinite scroll\" interface), then you can tag the prop value using the `inertia_merge/1` helper:\n\n```elixir\nconn\n|> assign_prop(:paginated_list, inertia_merge([\"a\", \"b\", \"c\"]))\n```\n\nMerge props can also accept deferred props:\n\n```elixir\nconn\n|> assign_prop(:paginated_list, inertia_defer(&calculate_next_page/0) |> inertia_merge())\n```","ref":"readme.html#merge-props"},{"type":"extras","title":"Shared data - Inertia.js Phoenix Adapter","doc":"To share data on every request, you can use the `assign_prop/2` function inside of a shared plug in your response pipeline. For example, suppose you have a `UserAuth` plug responsible for fetching the currently-logged in user and you want to be sure all your Inertia components receive that user data. Your plug might look something like this:\n\n```elixir\ndefmodule MyApp.UserAuth do\n  import Inertia.Controller\n  import Phoenix.Controller\n  import Plug.Conn\n\n  def authenticate_user(conn, _opts) do\n    user = get_user_from_session(conn)\n\n    # Here we are storing the user in the conn assigns (so\n    # we can use it for things like checking permissions later on),\n    # AND we are assigning a serialized represention of the user\n    # to our Inertia props.\n    conn\n    |> assign(:user, user)\n    |> assign_prop(:user, serialize_user(user))\n  end\n\n  # ...\nend\n```\n\nAnywhere this plug is used, the serialized `user` prop will be passed to the Inertia component.","ref":"readme.html#shared-data"},{"type":"extras","title":"Validations - Inertia.js Phoenix Adapter","doc":"Validation errors follow some specific conventions to make wiring up with Inertia's form helpers seamless. The `errors` prop is managed by this library and is always included in the props object for Inertia components. (When there are no errors, the `errors` prop will be an empty object).\n\nThe `assign_errors` function is how you tell Inertia what errors should be represented on the front-end. By default, you can either pass an `Ecto.Changeset` struct or a bare map to the `assign_errors` function. For other error data types, you may implement the `Inertia.Errors` protocol (see the `Inertia.Errors` module docs for more information).\n\n```elixir\ndef update(conn, params) do\n  case MyApp.Settings.update(params) do\n    {:ok, _settings} ->\n      conn\n      |> put_flash(:info, \"Settings updated\")\n      |> redirect(to: ~p\"/settings\")\n\n    {:error, changeset} ->\n      conn\n      |> assign_errors(changeset)\n      |> redirect(to: ~p\"/settings\")\n  end\nend\n```\n\nThe `assign_errors` function will automatically convert the changeset errors into a shape compatible with the client-side adapter. Since Inertia.js expects a flat map of key-value pairs, the error serializer will flatten nested errors down to compound keys:\n\n```javascript\n{\n  \"name\" => \"can't be blank\",\n\n  // Nested errors keys are flattened with a dot separator (`.`)\n  \"team.name\" => \"must be at least 3 characters long\",\n\n  // Nested arrays are zero-based and indexed using bracket notation (`[0]`)\n  \"items[1].price\" => \"must be greater than 0\"\n}\n```\n\nErrors are automatically preserved across redirects, so you can safely respond with a redirect back to page where the form lives to display form errors.\n\nIf you need to construct your own map of errors (rather than pass in a changeset), be sure it's a flat mapping of atom (or string) keys to string values like this:\n\n```elixir\nconn\n|> assign_errors(%{\n  name: \"Name can't be blank\",\n  password: \"Password must be at least 5 characters\"\n})\n```","ref":"readme.html#validations"},{"type":"extras","title":"Flash messages - Inertia.js Phoenix Adapter","doc":"This library automatically includes Phoenix flash data in Inertia props, under the `flash` key.\n\nFor example, given the following controller action:\n\n```elixir\ndef update(conn, params) do\n  case MyApp.Settings.update(params) do\n    {:ok, _settings} ->\n      conn\n      |> put_flash(:info, \"Settings updated\")\n      |> redirect(to: ~p\"/settings\")\n\n    {:error, changeset} ->\n      conn\n      |> assign_errors(changeset)\n      |> redirect(to: ~p\"/settings\")\n  end\nend\n```\n\nWhen Inertia (or the browser) redirects to the `/settings` page, the Inertia component will receive the flash props:\n\n```javascript\n{\n  \"component\": \"...\",\n  \"props\": {\n    \"flash\": {\n      \"info\": \"Settings updated\"\n    },\n    // ...\n  }\n}\n```","ref":"readme.html#flash-messages"},{"type":"extras","title":"CSRF protection - Inertia.js Phoenix Adapter","doc":"This library automatically sets the `XSRF-TOKEN` cookie for use by the Axios client on the front-end. Since Phoenix expects to receive the CSRF token via the `x-csrf-token` header, you'll need to configure Axios in your front-end JavaScript to use that header name:\n\n```javascript\n// assets/js/app.js\n\nimport axios from \"axios\";\naxios.defaults.xsrfHeaderName = \"x-csrf-token\";\n\n// the rest of your Inertia client code...\n```","ref":"readme.html#csrf-protection"},{"type":"extras","title":"History - Inertia.js Phoenix Adapter","doc":"**Requires Inertia v2.x on the client-side**.","ref":"readme.html#history"},{"type":"extras","title":"Encryption - Inertia.js Phoenix Adapter","doc":"If your page props contain sensitive data (such as information about the currently-authenticated user), you can opt to encrypt the history data that's cached in the browser.\n\n```elixir\nconn\n|> encrypt_history()\n```\n\nYou can also enable history encryption globally in your application config:\n\n```elixir\nconfig :inertia,\n  history: [encrypt: true]\n```","ref":"readme.html#encryption"},{"type":"extras","title":"Clearing history - Inertia.js Phoenix Adapter","doc":"To instruct the client to clear this history (for example, when a user logs out), you can use the `clear_history/1` helper when building your response.\n\n```elixir\nconn\n|> clear_history()\n```","ref":"readme.html#clearing-history"},{"type":"extras","title":"Testing - Inertia.js Phoenix Adapter","doc":"The `Inertia.Testing` module includes helpers for testing your Inertia controller responses, such as the `inertia_component/1` and `inertia_props/1` functions.\n\n\n```elixir\nuse MyAppWeb.ConnCase\n\nimport Inertia.Testing\n\ndescribe \"GET /\" do\n  test \"renders the home page\", %{conn: conn} do\n    conn = get(\"/\")\n    assert inertia_component(conn) == \"Home\"\n    assert %{user: %{id: 1}} = inertia_props(conn)\n  end\nend\n```\n\n```elixir\nuse MyAppWeb.ConnCase\n\nimport Inertia.Testing\n\ndescribe \"POST /users\" do\n  test \"fails when name empty\", %{conn: conn} do\n    conn = post(\"/users\", %{\"name\" => \"\"})\n\n    assert %{user: %{id: 1}} = inertia_props(conn)\n    assert redirected_to(conn) == ~p\"/users\"\n    assert inertia_errors(conn) == %{\"name\" => \"can't be blank\"}\n  end\nend\n```\n\nWe recommend importing `Inertia.Testing` in your `ConnCase` helper, so that it will be at the ready for all your controller tests:\n\n```elixir\ndefmodule MyApp.ConnCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      import Inertia.Testing\n\n      # ...\n    end\n  end\nend\n```","ref":"readme.html#testing"},{"type":"extras","title":"Server-side rendering - Inertia.js Phoenix Adapter","doc":"The Inertia.js client library comes with with server-side rendering (SSR) support, which means you can have your Inertia-powered client hydrate HTML that has been pre-rendered on the server (instead of performing the initial DOM rendering).\n\n> [!NOTE]\n> The steps for enabling SSR in Phoenix are similar to other backend frameworks, but instead of running a separate Node.js server process to render HTML, this library spins up a pool of Node.js process workers to handle SSR calls and manages the state of those node processes from your Elixir process tree. This is mostly just an implementation detail that you don't need to be concerned about, but we'll highlight how our `ssr.js` script differs from the Inertia.js docs.","ref":"readme.html#server-side-rendering"},{"type":"extras","title":"Add a server-side rendering module - Inertia.js Phoenix Adapter","doc":"You'll need to create a JavaScript module that exports a `render` function to perform the actual server-side rendering of pages. For the purpose of these instructions, we'll assume you're using React. The steps would be similar for other front-end environments supported by Inertia.js, such as [Vue](https://github.com/CallumVass/inertia_vue) and [Svelte](https://github.com/tonydangblog/phoenix-inertia-svelte).\n\nSuppose your main `app.jsx` file looks something like this:\n\n```js\n// assets/js/app.jsx\n\nimport React from \"react\";\nimport { createInertiaApp } from \"@inertiajs/react\";\nimport { createRoot } from \"react-dom/client\";\n\ncreateInertiaApp({\n  resolve: async (name) => {\n    return await import(`./pages/${name}.jsx`);\n  },\n  setup({ App, el, props }) {\n    createRoot(el).render( );\n  },\n});\n```\n\nYou'll need to create a second JavaScript file (alongside your `app.jsx`) that exports a `render` function. Let's name it `ssr.jsx`.\n\n```js\n// assets/js/ssr.jsx\n\nimport React from \"react\";\nimport ReactDOMServer from \"react-dom/server\";\nimport { createInertiaApp } from \"@inertiajs/react\";\n\nexport function render(page) {\n  return createInertiaApp({\n    page,\n    render: ReactDOMServer.renderToString,\n    resolve: async (name) => {\n      return await import(`./pages/${name}.jsx`);\n    },\n    setup: ({ App, props }) =>  ,\n  });\n}\n```\n\nThis is similar to the server entry-point [documented here](https://inertiajs.com/server-side-rendering#add-server-entry-point), except we are simply **exporting a function called `render`**, instead of starting a Node.js server process.\n\nNext, configure esbuild to compile the `ssr.jsx` bundle.\n\n```diff\n  # config/config.exs\n\n  config :esbuild,\n    version: \"0.21.5\",\n    app: [\n      args: ~w(js/app.jsx --bundle --target=es2020 --outdir=../priv/static/assets --external:/fonts/* --external:/images/*),\n      cd: Path.expand(\"../assets\", __DIR__),\n      env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n    ],\n+   ssr: [\n+     args: ~w(js/ssr.jsx --bundle --platform=node --outdir=../priv --format=cjs),\n+     cd: Path.expand(\"../assets\", __DIR__),\n+     env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n+   ]\n```\n\nAdd the `ssr` build to the watchers in your dev environment, alongside the other asset watchers:\n\n```diff\n  # config/dev.exs\n  config :my_app, MyAppWeb.Endpoint,\n    # Binding to loopback ipv4 address prevents access from other machines.\n    # Change to `ip: {0, 0, 0, 0}` to allow access from other machines.\n    http: [ip: {127, 0, 0, 1}, port: 4000],\n    check_origin: false,\n    code_reloader: true,\n    debug_errors: true,\n    secret_key_base: \"4Z2yyTu6Uy8AM+MguG3oldEf4aIdswR2BsCm1OtqDK0lEv++T02KktRaXfMbC/Zs\",\n    watchers: [\n      esbuild: {Esbuild, :install_and_run, [:app, ~w(--sourcemap=inline --watch)]},\n+     ssr: {Esbuild, :install_and_run, [:ssr, ~w(--sourcemap=inline --watch)]},\n      tailwind: {Tailwind, :install_and_run, [:my_app, ~w(--watch)]}\n    ]\n```\n\nAdd the `ssr` build step to the asset build and deploy scripts.\n\n```diff\n  # mix.exs\n\n  defp aliases do\n    [\n      setup: [\"deps.get\", \"ecto.setup\", \"assets.setup\", \"assets.build\"],\n      \"ecto.setup\": [\"ecto.create\", \"ecto.migrate\", \"run priv/repo/seeds.exs\"],\n      \"ecto.reset\": [\"ecto.drop\", \"ecto.setup\"],\n      test: [\"ecto.create --quiet\", \"ecto.migrate --quiet\", \"test\"],\n      \"assets.setup\": [\"tailwind.install --if-missing\", \"esbuild.install --if-missing\"],\n-     \"assets.build\": [\"tailwind app\", \"esbuild app\"],\n+     \"assets.build\": [\"tailwind app\", \"esbuild app\", \"esbuild ssr\"],\n      \"assets.deploy\": [\n        \"tailwind app --minify\",\n        \"esbuild app --minify\",\n+       \"esbuild ssr\",\n        \"phx.digest\"\n      ]\n    ]\n  end\n```\n\nAs configured, this will place the generated `ssr.js` bundle into the `priv` directory. Since it's generated code, add it to your `.gitignore` file.\n\n```diff\n  # .gitignore\n\n+ /priv/ssr.js\n```","ref":"readme.html#add-a-server-side-rendering-module"},{"type":"extras","title":"Configuring your app for server-rendering - Inertia.js Phoenix Adapter","doc":"Now that you have a Node.js module capable of server-rendering your pages, youll need to tell the Inertia.js Phoenix library to perform SSR.\n\nFirst, add the `Inertia.SSR` module to your application's supervision tree.\n\n```diff\n  # lib/my_app/application.ex\n\n  defmodule MyApp.Application do\n    use Application\n\n    @impl true\n    def start(_type, _args) do\n      children = [\n        MyAppWeb.Telemetry,\n        MyApp.Repo,\n        {DNSCluster, query: Application.get_env(:MyApp, :dns_cluster_query) || :ignore},\n        {Phoenix.PubSub, name: MyApp.PubSub},\n        # Start the Finch HTTP client for sending emails\n        {Finch, name: MyApp.Finch},\n        # Start a worker by calling: MyApp.Worker.start_link(arg)\n        # {MyApp.Worker, arg},\n\n+       # Start the SSR process pool\n+       # You must specify a `path` option to locate the directory where the `ssr.js` file lives.\n+       {Inertia.SSR, path: Path.join([Application.app_dir(:my_app), \"priv\"])},\n\n        # Start to serve requests, typically the last entry\n        MyAppWeb.Endpoint,\n      ]\n```\n\nThen, update your config to enable SSR (if you'd like to enable it globally).\n\n```diff\n  # config/config.exs\n\n  config :inertia,\n    # The Phoenix Endpoint module for your application. This is used for building\n    # asset URLs to compute a unique version hash to track when something has\n    # changed (and a reload is required on the frontend).\n    endpoint: MyAppWeb.Endpoint,\n\n    # An optional list of static file paths to track for changes. You'll generally\n    # want to include any JavaScript assets that may require a page refresh when\n    # modified.\n    static_paths: [\"/assets/app.js\"],\n\n    # The default version string to use (if you decide not to track any static\n    # assets using the `static_paths` config). Defaults to \"1\".\n    default_version: \"1\",\n\n    # Enable server-side rendering for page responses (requires some additional setup,\n    # see instructions below). Defaults to `false`.\n-   ssr: false\n+   ssr: true\n\n    # Whether to raise an exception when server-side rendering fails (only applies\n    # when SSR is enabled). Defaults to `true`.\n    #\n    # Recommended: enable in non-production environments and disable in production,\n    # so that SSR failures will not cause 500 errors (but instead will fallback to\n    # CSR).\n    raise_on_ssr_failure: config_env() != :prod\n```","ref":"readme.html#configuring-your-app-for-server-rendering"},{"type":"extras","title":"Installing Node.js in your production - Inertia.js Phoenix Adapter","doc":"You need to have Node.js installed in your production server environment, so that we can call the SSR script when serving pages. These steps assume you are deploying your application using a Dockerfile and releases.\n\nIf you haven't installed node into your runner image, add the following command to your Dockerfile (after the `FROM ${RUNNER_IMAGE}` step).\n\n```diff\n  FROM ${RUNNER_IMAGE}\n\n  # install curl (and a few other packages)\n  RUN apt-get update -y && \\\n-     apt-get install -y libstdc++6 openssl libncurses5 locales ca-certificates && \\\n+     apt-get install -y libstdc++6 openssl curl libncurses5 locales ca-certificates && \\\n      apt-get clean && rm -f /var/lib/apt/lists/*_*\n\n  # install Node.js\n+ RUN curl -fsSL https://deb.nodesource.com/setup_x.x | bash - && \\\n+    apt-get update && \\\n+    apt-get install -y nodejs\n\n  # ...\n\n  ENV MIX_ENV=\"prod\"\n\n  # ensure node is running in production mode\n+ ENV NODE_ENV=\"production\"\n```\n\n> [!IMPORTANT]\n> **Be sure to set `NODE_ENV=production`**, so that the SSR script is cached in memory. Otherwise, your page rendering times will be very slow!","ref":"readme.html#installing-node-js-in-your-production"},{"type":"extras","title":"Client side hydration - Inertia.js Phoenix Adapter","doc":"[Follow the instructions from the Inertia.js docs](https://inertiajs.com/server-side-rendering#client-side-hydration) for updating your client-side code to hydrate the pre-rendered HTML coming from the server.\n\nUsing our example React script from above, the adaptation looks like this:\n\n```diff\n  // assets/js/app.jsx\n\n  import React from \"react\";\n  import { createInertiaApp } from \"@inertiajs/react\";\n- import { createRoot } from \"react-dom/client\";\n+ import { hydrateRoot } from \"react-dom/client\";\n\n  createInertiaApp({\n    resolve: async (name) => {\n      return await import(`./pages/${name}.jsx`);\n    },\n    setup({ App, el, props }) {\n-     createRoot(el).render( );\n+     hydrateRoot(el,  );\n    },\n  });\n```\n\n---\n\nMaintained by the team at [SavvyCal](https://savvycal.com)","ref":"readme.html#client-side-hydration"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"2.4.0 - Changelog","doc":"","ref":"changelog.html#2-4-0"},{"type":"extras","title":"Added - Changelog","doc":"- Add `inertia_errors/1` test helper to fetch Inertia errors ([#43](https://github.com/inertiajs/inertia-phoenix/pull/43)).","ref":"changelog.html#added"},{"type":"extras","title":"2.3.0 - Changelog","doc":"","ref":"changelog.html#2-3-0"},{"type":"extras","title":"Added - Changelog","doc":"- Add a `force_inertia_redirect` plug function to instruct the client-side to always perform a full browser redirect when a redirect response is sent ([#35](https://github.com/inertiajs/inertia-phoenix/issues/35)).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Define an `Inertia.Errors` protocol with default implementations for `Ecto.Changeset` and `Map`.","ref":"changelog.html#changed"},{"type":"extras","title":"2.2.0 - Changelog","doc":"","ref":"changelog.html#2-2-0"},{"type":"extras","title":"Added - Changelog","doc":"- Add `preserve_case` helper to prevent auto-camelization of specified prop keys.\n- Add `Inertia.Controller.inertia_response?/1` helper to determine if a response is Inertia-rendered.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Ensure prop keys are compared in the proper casing (for partial reloads) when `camelize_props` is enabled.\n- Fix prop resolution for deferred/optional props.","ref":"changelog.html#fixed"},{"type":"extras","title":"2.1.0 - Changelog","doc":"","ref":"changelog.html#2-1-0"},{"type":"extras","title":"Fixed - Changelog","doc":"- Include new Inertia v2 attributes in the initial page object (`mergeProps`, `deferredProps`, `encryptHistory`, `clearHistory`).\n- Mark internal component functions in `Inertia.HTML` as private.","ref":"changelog.html#fixed"},{"type":"extras","title":"2.0.0 - Changelog","doc":"","ref":"changelog.html#2-0-0"},{"type":"extras","title":"Added - Changelog","doc":"- Add support new Inertia.js v2.0.0.\n  - Add `encrypt_history` function to instruct the client-side to encrypt the history entry.\n  - Add `clear_history` function to instruct the client-side to clear history.\n  - Add `inertia_optional` function, to replace the now-deprecated `inertia_lazy` function.\n  - Add `inertia_merge` function to instruct the client-side to merge the prop value with existing data.\n  - Add `inertia_defer` function to instruct the client-side to fetch the prop value immediately after initial page load.\n- Add helpers for testing Inertia-based controller responses via the `Inertia.Testing` module.\n- Add a `camelize_props` global config option and a `camelize_props` function (to use on a per-request basis) to automatically convert prop keys from snake case to camel case.\n- Accept an `ssr` option on the `render_inertia` function.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Update Phoenix LiveView to v1.0.\n- The errors serializer (for `Ecto.Changeset` structs) has been adjusted to better align with the behavior in the Laravel adapter in cases when there are **multiple validation errors for a single field**.\n\n**Old behavior for errors serializer**\n\nPreviously, the serializer would include each error under a separate key, with a `[0]` index suffix, like this:\n\n```javascript\n{\n  \"name[0]\": \"is too long\",\n  \"name[1]\": \"is not real\"\n}\n```\n\nWhile this retains maximal information about all the errors for a field, in practice it's difficult to target the right error records for display in the UI.\n\n**New behavior for errors serializer**\n\nNow, the serializer simply takes the _first error message_ and returns it under the field name, without any added suffix:\n\n```javascript\n{\n  \"name\": \"is too long\"\n}\n```","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Allow for external redirects from `PUT` / `PATCH` / `DELETE` requests ([#22](https://github.com/inertiajs/inertia-phoenix/pull/22))\n- Camelize prop names inside lists (e.g. `assign_prop(:items, [%{item_name: \"...\"}])`).","ref":"changelog.html#fixed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- The `inertia_lazy/1` function has been deprecated in favor of `inertia_optional/1`","ref":"changelog.html#deprecated"},{"type":"extras","title":"0.10.0 - Changelog","doc":"","ref":"changelog.html#0-10-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Remove unsupported dot-notation in partial requests (related to [inertiajs/inertia-laravel#641](https://github.com/inertiajs/inertia-laravel/pull/641))","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.9.0 - Changelog","doc":"","ref":"changelog.html#0-9-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix improper elimination of nested props when using only partials","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.8.0 - Changelog","doc":"","ref":"changelog.html#0-8-0"},{"type":"extras","title":"Features - Changelog","doc":"- Support unicode props (by using the `binary` flag on Node function calls)","ref":"changelog.html#features"},{"type":"extras","title":"0.7.0 - Changelog","doc":"","ref":"changelog.html#0-7-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix exception when assigning structs as prop values (like `DateTime`)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.6.0 - Changelog","doc":"","ref":"changelog.html#0-6-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Prevent overly greedy empty object elimination ([#14](https://github.com/inertiajs/inertia-phoenix/pull/14))","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.5.0 - Changelog","doc":"- Assign errors via an `assign_errors` helper ([#10](https://github.com/inertiajs/inertia-phoenix/issues/10))\n- Preserve assigned errors across redirects ([#10](https://github.com/inertiajs/inertia-phoenix/issues/10))\n- Set up external redirects properly for Inertia requests ([#11](https://github.com/inertiajs/inertia-phoenix/issues/11))\n- Pass CSRF tokens via cookies ([#12](https://github.com/inertiajs/inertia-phoenix/issues/12)) \n- Forward flash contents across forced refreshes ([#13](https://github.com/inertiajs/inertia-phoenix/issues/13))\n- Automatically pass Phoenix flash data via the `flash` prop","ref":"changelog.html#0-5-0"},{"type":"extras","title":"0.4.0 - Changelog","doc":"- Support for partial reloads ([#6](https://github.com/inertiajs/inertia-phoenix/issues/6))\n- Support lazy data evaluation ([#7](https://github.com/inertiajs/inertia-phoenix/issues/7))","ref":"changelog.html#0-4-0"},{"type":"extras","title":"0.3.0 - Changelog","doc":"- Add `raise_on_ssr_failure` configuration","ref":"changelog.html#0-3-0"},{"type":"extras","title":"0.2.0 - Changelog","doc":"- Add SSR support\n- Add `<.inertia_head>` component for rendering head elements provided by SSR","ref":"changelog.html#0-2-0"},{"type":"extras","title":"0.1.0 - Changelog","doc":"- Initial release","ref":"changelog.html#0-1-0"},{"type":"extras","title":"License","doc":"# License\n\nMIT License\n\nCopyright (c) 2024 SavvyCal, Inc.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}